%Calculate axial and transverse free space beam profiles
%Copyright (C) 2018 Peter Munro
%
%This program is free software: you can redistribute it and/or modify
%it under the terms of the GNU Lesser Public License as published by
%the Free Software Foundation, either version 3 of the License, or
%(at your option) any later version.
%
%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU Lesser Public License for more details.
%    
%You should have received a copy of the GNU Lesser Public License
%along with this program.  If not, see <http://www.gnu.org/licenses/>.

%set to 1 if fields should be recalculated
recompute = 0;

%free space refractive index
nvec_fs = 1.;
hvec_fs = [];

%numerical aperture of OCT system objective
NA = 3.5/36;
%wavelength

%number of points in theta and phi to integrate over
ntheta = 100;
nphi = 100;

%First calculate a transverse profile
xmin = -20e-6;
xmax =  20e-6;
Nx = 101;

ymin = xmin;
ymax = xmax;
Ny = Nx;

zmin = 0;
zmax = 0;
Nz = 1;

x = linspace(xmin,xmax,Nx);
y = linspace(ymin,ymax,Ny);
z = linspace(zmin,zmax,Nz);

[X,Y,Z] = ndgrid(x,y,z);
vertices = [X(:) Y(:) Z(:)];

if recompute
    [Efs_trans,~] = focstratfield_general_pol(vertices,nvec_fs,hvec_fs,NA,lambda,ntheta,nphi,@telesto_pol);
    save cffsvars Efs_trans;
else
    load cffsvars;
end

figure(1);clf;
for i=1:3
    subplot(1,3,i);
    h = imagesc(x*1e6,y*1e6,abs(reshape(Efs_trans(:,i),size(X))).');
    axis equal;
    axis tight;
    hx = xlabel('x (\mum)');
    hy = ylabel('y (\mum)');
    ht = title(sprintf('|E_%c|','x'+i-1));
end

%Now calculate an axial profile
xmin = -100e-6;
xmax =  100e-6;
Nx = 201;

ymin = 0;
ymax = 0;
Ny = 1;

zmin = -1000e-6;
zmax = 1000e-6;
Nz = 201;

x = linspace(xmin,xmax,Nx);
y = linspace(ymin,ymax,Ny);
z = linspace(zmin,zmax,Nz);

[X,Y,Z] = ndgrid(x,y,z);
vertices = [X(:) Y(:) Z(:)];

if recompute
    [Efs_ax,~] = focstratfield_general_pol(vertices,nvec_fs,hvec_fs,NA,lambda,ntheta,nphi,@telesto_pol);
    save cffsvars Efs_ax -append;
else
    load cffsvars;
end


figure(2);clf;
for i=1:2
    subplot(1,2,i);
    imagesc(x*1e3,z*1e3,abs(squeeze(reshape(Efs_ax(:,2*i-1),size(X)))).');
    axis equal;
    axis tight;
    hx = xlabel('x (mm)');
    hy = ylabel('y (mm)');
    ht = title(sprintf('|E_%c|','x'+2*i-2));
end

    
