%Example where an A-scan is calculated for a series of plane
%reflectors arranged along the optical axis
%Copyright (C) 2018 Peter Munro
%
%This program is free software: you can redistribute it and/or modify
%it under the terms of the GNU Lesser Public License as published by
%the Free Software Foundation, either version 3 of the License, or
%(at your option) any later version.
%
%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU Lesser Public License for more details.
%    
%You should have received a copy of the GNU Lesser Public License
%along with this program.  If not, see <http://www.gnu.org/licenses/>.

%set to 1 if fields should be recalculated
recompute = 0;

%free space refractive index
nvec_fs = 1.;
hvec_fs = [];

%separation of reflectors
pitch_ref = 25e-6;
%width of reflectors
width_ref = 100e-9;
%construct hvec
hvec_ml = sort( [(0:pitch_ref:500e-6)-width_ref/2 (0:pitch_ref:500e-6)+width_ref/2]);
%employ weak refractive inded contrast 
n0 = 1;
n1 = 1.01;
nvec_ml = [repmat([n0 n1],[1 numel(hvec_ml)/2]) n0];
%to calculate the reference arm light
hvec_ref = [-50e-6];
nvec_ref = [1 10];

%numerical aperture of OCT system objective
NA = 3.5/36;
%wavenumbers at which to sample spectrum
k_vec = linspace( 2*pi/1420e-9, 2*pi/1180e-9, 2048);

%number of points in theta and phi to integrate over
ntheta = 100;
nphi = 100;

%assume the that beam remains rotationally symmetric, so only need
%to perform a 1-dimensional integration to calculate coupling into
%the fiber
xmin = 0;
xmax = 60e-6;
Nx = 101;

ymin = 0;
ymax = 0;
Ny = 1;

zmin = hvec_ref(1)-1e-9;
zmax = zmin;
Nz = 1;

[x,wx] = gauss_legendre(xmin,xmax,Nx);
y = linspace(ymin,ymax,Ny);
z = linspace(zmin,zmax,Nz);

[X,Y,Z] = ndgrid(x,y,z);
[WX,~,~] = ndgrid(wx,y,z);

vertices = [X(:) Y(:) Z(:)];
Esam = zeros(numel(k_vec),Nx);Eref = zeros(numel(k_vec),Nx);Einc = zeros(numel(k_vec),Nx);
if recompute
    parfor ik=1:numel(k_vec)
	[~,Esam_temp] = focstratfield_general_pol(vertices,nvec_ml,hvec_ml,NA,2*pi/k_vec(ik),ntheta,nphi,@telesto_pol);
	Esam(ik,:) = Esam_temp(:,1);
	
	[Einc_temp,~] = focstratfield_general_pol(vertices,nvec_fs,hvec_fs,NA,2*pi/k_vec(ik),ntheta,nphi,@telesto_pol);
	Einc(ik,:) = Einc_temp(:,1);
	
	[~,Eref_temp] = focstratfield_general_pol(vertices,nvec_ref,hvec_ref,NA,2*pi/k_vec(ik),ntheta,nphi,@telesto_pol);
	Eref(ik,:) = Eref_temp(:,1);
	
	fprintf(1,'Done %d/%d\n',ik,numel(k_vec));
    end
    save goe02vars Einc Eref Esam;
else
    load goe02vars;
end

%Perform the OCT reconstruction
Ik_sam = 2*pi*( (wx.*x)*(Einc.*Esam).' );
Ik_ref = 2*pi*( (wx.*x)*(Einc.*Eref).' );

W = hann(numel(Ik_sam));
OCT = ifft([2*real(Ik_sam.*conj(Ik_ref)).*W.' zeros(1,5*numel(W))]);
zbar = (0:(numel(OCT)-1))/numel(OCT)/diff(k_vec(900:901)/2/pi)/2;

%Now check the result
[x,wx] = gauss_legendre(0,150e-6,200);
y = linspace(ymin,ymax,Ny);
z = linspace(zmin,zmax,Nz);

[X,Y,Z] = ndgrid(x,y,z);
[WX,~,~] = ndgrid(wx,y,z);

zpsf = linspace(0,hvec_ml(end),150);
lambda=1300e-9;
if recompute
    Izpsf = zeros(1,numel(zpsf));Izpsf_mirr = zeros(1,numel(zpsf));
    Einc_acc = zeros(numel(zpsf),numel(X));
    Eref_acc = zeros(numel(zpsf),numel(X));
    vertices = [X(:) Y(:) ones(size(Z(:)))*(-1e-10)];
    [Einc0] = focstratfield_general_pol(vertices,nvec_fs,hvec_fs,NA,lambda,ntheta,nphi,@telesto_pol);
    
    parfor iz=1:numel(zpsf)
	vertices = [X(:) Y(:) ones(size(Z(:)))*zpsf(iz)];
	[Einc_temp] = focstratfield_general_pol(vertices,nvec_fs,hvec_fs,NA,lambda,ntheta,nphi,@telesto_pol);
	Izpsf(iz) = abs(2*pi*( (wx.*x)*(Einc_temp(:,1).^2) ));
	Einc_acc(iz,:)  = Einc_temp(:,1);
	
	vertices = [X(:) Y(:) ones(size(Z(:)))*0];
	nvec_mirr = [1 10000];
	hvec_mirr = zpsf(iz)+1e-10;
	[Eref_temp] = focstratfield_general_pol(vertices,nvec_mirr,hvec_mirr,NA,lambda,ntheta,nphi,@telesto_pol);
	Eref_acc(iz,:)  = Eref_temp(:,1);
	Izpsf_mirr(iz) =  abs(2*pi*( (wx.*x)*(Einc0(:,1).*Eref_temp(:,1)) ));
	fprintf(1,'Done %d/%d\n',iz,numel(zpsf));
    end 
    save Izpsf Izpsf;
else
    load Izpsf;
end

fs = 16;
figure(1);clf;
hp1 = plot(1e3*(zbar+hvec_ref),abs(OCT)/max(abs(OCT)));
hold on;
hp2 = plot(zpsf*1e3,Izpsf/Izpsf(1));
axis([-.05 .550 -0.05 1.05]);

set([hp1 hp2],'LineWidth',2);
hx = xlabel('$z$ (mm)');
hy = ylabel('$|$OCT$|$');
set(gca,'FontSize',16);
set([hx hy],'Interpreter','latex');
