%Example where an A-scan is calculated of water sandwiched between
%two glass interfaces
%Copyright (C) 2018 Peter Munro
%
%This program is free software: you can redistribute it and/or modify
%it under the terms of the GNU Lesser Public License as published by
%the Free Software Foundation, either version 3 of the License, or
%(at your option) any later version.
%
%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU Lesser Public License for more details.
%    
%You should have received a copy of the GNU Lesser Public License
%along with this program.  If not, see <http://www.gnu.org/licenses/>.

%requires that focstratfield_general_pol.m and gauss_legendre.m are
%in the current path 

%Refractive index of water data
%Retrieved from http://omlc.org/spectra/water/data/segelstein81_index.txt
water_data = [1.099E+00	1.319865	1.708E-06
1.109E+00	1.319678	2.038E-06
1.119E+00	1.319488	2.628E-06
1.130E+00	1.319296	3.869E-06
1.140E+00	1.319103	5.951E-06
1.151E+00	1.318909	9.306E-06
1.159E+00	1.318763	1.069E-05
1.169E+00	1.318566	1.120E-05
1.180E+00	1.318366	1.160E-05
1.191E+00	1.318162	1.181E-05
1.199E+00	1.318008	1.199E-05
1.211E+00	1.317799	1.191E-05
1.219E+00	1.317641	1.179E-05
1.230E+00	1.317427	1.160E-05
1.239E+00	1.317263	1.139E-05
1.250E+00	1.317042	1.100E-05
1.259E+00	1.316873	1.079E-05
1.271E+00	1.316645	1.090E-05
1.279E+00	1.316470	1.139E-05
1.291E+00	1.316233	1.221E-05
1.300E+00	1.316052	1.400E-05
1.309E+00	1.315868	1.639E-05
1.321E+00	1.315618	1.912E-05
1.330E+00	1.315425	2.251E-05
1.340E+00	1.315228	2.849E-05
1.349E+00	1.315031	4.047E-05
1.361E+00	1.314760	4.505E-05
1.371E+00	1.314547	5.804E-05
1.380E+00	1.314329	7.802E-05
1.390E+00	1.314104	1.060E-04
1.400E+00	1.313871	1.530E-04
1.409E+00	1.313671	2.540E-04
1.419E+00	1.313518	3.197E-04
1.429E+00	1.313373	3.538E-04
1.439E+00	1.313220	3.629E-04
1.449E+00	1.313055	3.637E-04
1.459E+00	1.312888	3.604E-04
1.469E+00	1.312715	3.387E-04
1.479E+00	1.312525	3.018E-04
1.489E+00	1.312318	2.659E-04
1.500E+00	1.312093	2.248E-04];

%central wavelength
lambda0 = 1300e-9;

%find the phase refractive index at the central wavelength
np = interp1(water_data(:,1)*1e-6,water_data(:,2),lambda0,'pchip');

%numerically differentiate the phase refractive index
dlambda = 10e-9;
dnp_dlambda = (interp1(water_data(:,1)*1e-6,water_data(:,2), ...
		       lambda0+dlambda/2,'pchip') - interp1(water_data(:,1)*1e-6,water_data(:,2),lambda0-dlambda/2,'pchip') )/dlambda;

%groupd refractive index at the central wavelength
ng = np - lambda0*dnp_dlambda;

%refractive index of glass
n0 = 1.525;

%setup the multilayer structures
nvec_fs = n0;
hvec_fs = [];

hvec_ref = -100e-6;
nvec_ref = [n0 1e4];

Nref = 2;
W_water = 0.5e-3;
hvec_ml = [0 W_water];

%numerical aperture of OCT objective
NA = 3.5/36;

%vector of wavenumbers
k_vec = linspace( 2*pi/1420e-9, 2*pi/1180e-9, 2048);

%number of points of integration over theta and phi
ntheta = 100;
nphi = 100;


xmin = 0;
xmax = 60e-6;
Nx = 101;

ymin = 0;
ymax = 0;
Ny = 1;

zmin = hvec_ref(1)-1e-9;
zmax = zmin;
Nz = 1;

%assume the that beam remains rotationally symmetric, so only need
%to perform a 1-dimensional integration to calculate coupling into
%the fiber
[x,wx] = gauss_legendre(xmin,xmax,Nx);
y = linspace(ymin,ymax,Ny);
z = linspace(zmin,zmax,Nz);

[X,Y,Z] = ndgrid(x,y,z);
[WX,~,~] = ndgrid(wx,y,z);

vertices = [X(:) Y(:) Z(:)];

%Light in the sample space
Esam = zeros(numel(k_vec),Nx);
%Light reflected by reference mirror
Eref = zeros(numel(k_vec),Nx);
%Light incident upon sample and reference mirror
Einc = zeros(numel(k_vec),Nx);

if 1
    parfor ik=1:numel(k_vec)
    n_water = interp1(water_data(:,1)*1e-6,water_data(:,2),2*pi/k_vec(ik),'spline');
	%nvec_ml = [n0 repmat([n_water n0],[1 Nref]) n0];
	nvec_ml = [n0  n_water  n0];
	
	[~,Esam_temp] = focstratfield_general_pol(vertices,nvec_ml,hvec_ml,NA,2*pi/k_vec(ik),ntheta,nphi,@telesto_pol);
	Esam(ik,:) = Esam_temp(:,1);
	
	[Einc_temp,~] = focstratfield_general_pol(vertices,nvec_fs,hvec_fs,NA,2*pi/k_vec(ik),ntheta,nphi,@telesto_pol);
	Einc(ik,:) = Einc_temp(:,1);
	
	[~,Eref_temp] = focstratfield_general_pol(vertices,nvec_ref,hvec_ref,NA,2*pi/k_vec(ik),ntheta,nphi,@telesto_pol);
	Eref(ik,:) = Eref_temp(:,1);
	
	fprintf(1,'Done %d/%d\n',ik,numel(k_vec));
    end
    save goe05vars Einc Eref Esam;
else
    load goe05vars;
end

%Perform the OCT reconstruction
alpha = 0.1;
Ik_sam = 2*pi*( (wx.*x)*(Einc.*Esam).' )*alpha;
Ik_ref = 2*pi*( (wx.*x)*(Einc.*Eref).' );


W = hann(numel(Ik_sam));
OCT = ifft([2*real(Ik_sam.*conj(Ik_ref)).*W.' zeros(1,5*numel(W))]);
%OCT = ifft([2*abs(Ik_sam).^2.*W.' zeros(1,5*numel(W))]);
zbar = (0:(numel(OCT)-1))/numel(OCT)/diff(k_vec(900:901)/2/pi)/2;


n_water = interp1(water_data(:,1)*1e-6,water_data(:,2),1300e-9);
zbar_water = (0:(numel(OCT)-1))/numel(OCT)/diff(k_vec(900:901)/2/pi)/2/ng;

fs = 16;
figure(1);clf;
hp1 = semilogy(1e3*(zbar+hvec_ref*n0)/ng,(abs(OCT)/max(abs(OCT))));
axis([-.050 1.050 3.6241e-05 1.05]);
set(hp1,'LineWidth',2);
set(gca,'FontSize',fs);
set(gca,'XTick',[0 0.5 1]);
grid on;
hx = xlabel('$z$ (mm)');
hy = ylabel('$|$OCT$|$');
set([hx hy],'Interpreter','latex');

figure(2);clf;
hp1 = plot(1e6*(zbar+hvec_ref*n0)/ng,(abs(OCT)/max(abs(OCT))));
hold on;
A1 = interp1((zbar+hvec_ref*n0)/ng,abs(OCT)/max(abs(OCT)),5e-4);
A2 = interp1((zbar+hvec_ref*n0)/ng,abs(OCT)/max(abs(OCT)),10e-4);
hp2 = plot(1e6*(zbar+hvec_ref*n0)/ng-500,(abs(OCT)/max(abs(OCT)))/A1);
hp3 = plot(1e6*(zbar+hvec_ref*n0)/ng-1000,(abs(OCT)/max(abs(OCT)))/A2);
set([hp1 hp2 hp3],'LineWidth',2);
axis([-15 15 -0.05 1.05]);
set(gca,'FontSize',fs);
grid on;

hx = xlabel('$z-z_{i}$ ($\mu$m)');
hy = ylabel('$|$OCT$|$');
set([hx hy],'Interpreter','latex');
hl = legend('$z_{1}=0$','$z_{2}=0.5$mm','$z_{3}=1$mm');
set(hl,'Interpreter','latex');
