%Calculate the field distribution that results when focussing a
%radially polarized Bessel-Gauss beam from a material with
%refractive index 1.518 into air. Follows:
%D. P. Biss and T. G. Brown, �Cylindrical vector beam focusing
%through a dielectric interface,� Optics Express 9, 490-7 (2001).
%
%and
%
%A. van de Nes, L. Billy, S. Pereira, and J. Braat, �Calculation of
%the vectorial field distribution in a stratified focal region of a
%high numerical aperture imaging system,� Optics Express 12,
%1281-1293 (2004). 
%Copyright (C) 2018 Peter Munro
%
%This program is free software: you can redistribute it and/or modify
%it under the terms of the GNU Lesser Public License as published by
%the Free Software Foundation, either version 3 of the License, or
%(at your option) any later version.
%
%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU Lesser Public License for more details.
%    
%You should have received a copy of the GNU Lesser Public License
%along with this program.  If not, see <http://www.gnu.org/licenses/>.

%wavelength
lambda = 400e-9;

%setup grid upon which to evaluate the field
x = linspace(-4*lambda,4*lambda,201);
y = 0;
z = linspace(-4*lambda,4*lambda,201);
[X,Y,Z] = ndgrid(x,y,z);
vertices = [X(:) Y(:) Z(:)];

%a single interface
nvec = [1.518 1.];
hvec = 0;

%numerical aperture
NA = 1.4;

%number of points in theta and phi to integrate over
ntheta = 500;
nphi = 500;

if 0
    addpath('/home/pmunro/code/focussed_multilayer');
    [Ep,Em] = focstratfield_general_pol(vertices,nvec,hvec,NA,lambda,ntheta,nphi,@besselgauss);
    rmpath('/home/pmunro/code/focussed_multilayer');
    save gfd06vars Ep Em;
else
    load gfd06vars;
end
E = Ep + Em;

figure(1);clf;
imagesc(x/lambda,z/lambda, log10(abs(squeeze(reshape(E(:,3),size(X))).').^2));
ca=caxis;
caxis([ca(2)-4 ca(2)]);
caz=caxis;
axis equal;
colormap hot;

figure(2);clf;
imagesc(x/lambda,z/lambda, log10(abs(squeeze(reshape(E(:,1),size(X))).').^2));
ca=caxis;
caxis([ca(2)-4 ca(2)]);
caxis(caz);
axis equal;
colormap hot;
